function boundary = mqboundary(u,p,varargin)
% This function computes the boundary of a bivariate quantile as described
% in Coblenz et al (2018).
%
% call: boundary = mqboundary(u,p[,ploton])
%
% input     u                   - nx2 matrix of pseudo-observations
%           p                   - level at which boundary is computed;
%                                 note: this is the value of the estimated
%                                 inverse Kendall distribution function of
%                                 the desired probability level - not the
%                                 probability level itself!
%           ploton (optional)   - controls whether a plot is produced or
%                                 not; default value: 1
%
% output    boundary            - dx2 matrix of coordinates of the quantile
%                                 set boundary
%
% References:
% Coblenz, Dyckerhoff, Grothe (2018), Nonparametric Estimation of 
% Multivariate Quantiles, Environmetrics 29(2): 1-23
%
% updated: 15.03.2018, MC - function now deals better with tied data
%          24.05.2018, MC - improved input parsing
%
% Author: Maximilian Coblenz, 2018
% This code is released under the 3-clause BSD license.
% 

% some parsing and sanity checks
ip = inputParser;
ip.addRequired('u',@ismatrix);
ip.addRequired('p',@isscalar);
ip.addOptional('ploton',@isscalar);
ip.parse(u,p,varargin{:});

if (p < 0 || p > 1)
    error('the quantile level must be between 0 and 1');
end

% initialization of variables
if nargin < 3    
    ploton = 1;   
else    
    ploton = varargin{1};   
end

if floor(size(u,1)*p) == 0   
    np=floor(size(u,1)*p) + 1;   
else     
    np=floor(size(u,1)*p);   
end

u1sort = sort(u(:,1));
boundary = [u1sort(np) 1];

% construct boundary
flag = 1;
ldummy = 1;
while ldummy == 1
    
    if (flag == 1) % 'search largest y'-branch
        
        aux = max(u(u(:,1)<=boundary(end,1) & u(:,2)<boundary(end,2),2));
        boundary = [boundary; [boundary(end,1) aux]];
        flag = 0;
        
        if (sum(u(:,1)>boundary(end,1) & u(:,2)<boundary(end,2)) == 0) % check, if we are finished
            
            boundary = [boundary; [1 boundary(end,2)]];
            ldummy = 0;
            
        end
        
    else % 'search largest x'-branch
        
        aux = min(u(u(:,1)>boundary(end,1) & u(:,2)<=boundary(end,2),1));
        boundary = [boundary; [aux boundary(end,2)]];
        flag = 1;
        
    end
            
end % while

% plot
if ploton
    
    figure
    plot(u(:,1),u(:,2),'.k')
    hold on
    plot(boundary(:,1),boundary(:,2),'-b')
    hold off
    
end


end