function [S_l,S_u,mq,kinv,bn] = mqconfreg_mp(u,p,alpha,varargin)
% This function computes the confidence region according to the approach by
% Mammen & Polonik (2013) as extended in Coblenz et al (2018) for bivariate 
% data.
%
% call: [S_l,S_u,mq,kinv,bn] = mqconfreg_mp(u,p,alpha[,ploton])
%
% input     u                   - nx2 matrix of pseudo-observations
%           p                   - level at which multivariate quantile is 
%                                 computed; i.e., the 100%*p-quantile
%           alpha               - determines the confidence level 1-alpha;
%                                 i.e., for 90% confidence level enter 
%                                 alpha = 0.1
%           ploton (optional)   - controls whether a plot is produced or
%                                 not; default value: 1
%
% output    S_l                 - dlx2 matrix of coordinates of the lower
%                                 confidence region boundary
%           S_u                 - dux2 matrix of coordinates of the upper
%                                 confidence region boundary
%           boundary            - dx2 matrix of coordinates of the
%                                 multivariate quantile boundary
%           kinv                - estimated inverse of Kendall distribution
%                                 function
%           bn                  - estimated quantile of the supremum
%                                 distance
%
% functions needed: ecopula, mqboundary, pobs
%
% References:
% Mammen, Polonik (2013), Confidence regions for level sets, Journal of
% Multivariate Analysis 122: 202-214. 
% Coblenz, Dyckerhoff, Grothe (2018), Confidence Regions for Multivariate 
% Quantiles. Water, 10: 996.
%
% updated: 24.05.2018
%
% Author: Maximilian Coblenz, 2018
% This code is released under the 3-clause BSD license.
% 

% some parsing and sanity checks
ip = inputParser;
ip.addRequired('u',@ismatrix);
ip.addRequired('p',@isscalar);
ip.addRequired('alpha',@isscalar);
ip.addOptional('ploton',@isscalar);
ip.parse(u,p,alpha,varargin{:});

if (min(u(:)) < 0 || max(u(:)) > 1) % rank-transform data
    
    u = pobs(u);
    
end

if (p < 0 || p > 1)
    error('the quantile level must be between 0 and 1');
end

if (alpha < 0 || alpha > 1)
    error('the confidence level must be between 0 and 1');
end

% initialization of variables
if nargin < 4    
    ploton = 1;   
else    
    ploton = varargin{1};   
end

n_bs = 1000; % number of bootstrap repetitions

betan = 1/sqrt(size(u,1)); % determines which points are evaluated
bootstrap_indices = randi(size(u,1),size(u,1),n_bs); % bootstrap indices
kinvbs = zeros(1,n_bs); 
Zstar = zeros(1,n_bs);

% compute inverse Kendall distribution function and multivariate quantile
Chat = ecopula(u);

p_grid = (1/size(u,1):1/size(u,1):1);
V_I = (Chat*(size(u,1)+1)-1)/(size(u,1)-1);
kendallfunc = sum(V_I*ones(1,length(p_grid))<=ones(size(u,1),1)*p_grid,1)/(size(u,1));
[~,b] = min(kendallfunc(:)+(kendallfunc(:)<p)*10000000);
kinv = p_grid(b);
mq = mqboundary(u,kinv,0);

% set, on which the supremum distance is evaluated
deltan = u(abs(Chat-kinv)<=betan,:);

% bootstrap distribution of supremum distance Zstar
for ii = 1:1:n_bs
    
    % generate bs sample
    databs = u(bootstrap_indices(:,ii),:);
    
    % estimate K^-1 on bs sample
    p_grid = (1/size(databs,1):1/size(databs,1):1);
    V_I = (ecopula(databs,databs)*(size(databs,1)+1)-1)/(size(databs,1)-1);
    kendallfunc = sum(V_I*ones(1,length(p_grid))<=ones(size(databs,1),1)*p_grid,1)/(size(databs,1));
    [~,b] = min(kendallfunc(:)+(kendallfunc(:)<p)*10000000);
    kinvbs(ii) = p_grid(b);
    
    % compute supremum distance on deltan
    Zstar(ii) = max(abs(ecopula(deltan,databs)-kinvbs(ii)-ecopula(deltan,u)+kinv));

end % ii

% compute bn, S_l and S_u
bn = quantile(Zstar,1-alpha);
S_l = mqboundary(u,max([0 kinv-bn]),0);
S_u = mqboundary(u,min([kinv+bn 1]),0);

% plot
if ploton
    
    figure
    plot(mq(:,1),mq(:,2),'-b')
    hold on
    plot(S_l(:,1),S_l(:,2),'-','color',[1 0.6 0]);
    plot(S_u(:,1),S_u(:,2),'-','color',[1 0.6 0]);
    plot(u(:,1),u(:,2),'.k')
    hold off
    xlim([0 1])
    ylim([0 1])
    xlabel('U_1')
    ylabel('U_2')
    
end


end